@@ -1,5 +1,4 @@
-""" PhotoExport class to export photos
-"""
+"""PhotoExport class to export photos"""
 
 from __future__ import annotations
 
@@ -9,6 +8,7 @@ import json
 import logging
 import os
 import pathlib
+import subprocess
 import typing as t
 from enum import Enum
 
@@ -20,6 +20,7 @@ from .exifwriter import ExifWriter, exif_options_from_options
 from .export_db import ExportDBTemp
 from .exportoptions import ExportOptions, ExportResults
 from .fileutil import FileUtil
+from .photoinfo_common import photoinfo_minify_dict
 from .phototemplate import RenderOptions
 from .platform import is_macos
 from .rich_utils import add_rich_markup_tag
@@ -59,6 +60,12 @@ if t.TYPE_CHECKING:
 # retry if download_missing/use_photos_export fails the first time (which sometimes it does)
 MAX_PHOTOSCRIPT_RETRIES = 3
 
+# threshold for consecutive AppleScript export errors before restarting Photos
+APPLESCRIPT_ERROR_THRESHOLD = 10
+
+# counter for tracking consecutive AppleScript export errors
+_consecutive_export_errors = 0
+
 logger = logging.getLogger("osxphotos")
 
 
@@ -137,6 +144,24 @@ class StagedFiles:
 class PhotoExporter:
     """Export a photo"""
 
+    @staticmethod
+    def _kill_photos_process():
+        """Kill the Photos app process to restart it"""
+        try:
+            # Use pkill to kill the main Photos application process
+            # Use -f to match the full path containing Photos.app
+            result = subprocess.run(
+                ["pkill", "-f", "Photos.app"], check=False, capture_output=True
+            )
+            if result.returncode == 0:
+                logger.debug(
+                    "Photos process killed due to consecutive AppleScript failures"
+                )
+            else:
+                logger.debug("No Photos process found to kill")
+        except Exception as e:
+            logger.warning(f"Failed to kill Photos process: {e}")
+
     def __init__(self, photo: "PhotoInfo", tmpdir: t.Optional[str] = None):
         self.photo = photo
         self._render_options = RenderOptions()
@@ -1045,7 +1070,7 @@ class PhotoExporter:
         # set data in the database
         with export_db.create_or_get_file_record(dest_str, self.photo.uuid) as rec:
             if rec.photoinfo:
-                last_data = json.loads(rec.photoinfo)
+                last_data = photoinfo_minify_dict(json.loads(rec.photoinfo))
                 # to avoid issues with datetime comparisons, list order
                 # need to deserialize from photo.json() instead of using photo.asdict()
                 current_data = json.loads(self.photo.json(shallow=True))
@@ -1067,7 +1092,7 @@ class PhotoExporter:
                 diff = json.dumps(diff, default=_json_default) if diff else None
             else:
                 diff = None
-            rec.photoinfo = self.photo.json(shallow=True)
+            rec.photoinfo = self.photo.json(shallow=False)
             rec.export_options = options.bit_flags
             # don't set src_sig as that is set above before any modifications by convert_to_jpeg or exiftool
             if not options.ignore_signature:
@@ -1140,6 +1165,17 @@ class PhotoExporter:
             has not been edited. This is due to how Photos Applescript interface works.
         """
 
+        global _consecutive_export_errors
+
+        # Check if we've hit the error threshold and need to restart Photos
+        if _consecutive_export_errors >= APPLESCRIPT_ERROR_THRESHOLD:
+            logger.warning(
+                f"AppleScript export has failed {_consecutive_export_errors} consecutive times, "
+                f"restarting Photos app"
+            )
+            self._kill_photos_process()
+            _consecutive_export_errors = 0
+
         dest = pathlib.Path(dest)
         if not dest.is_dir():
             raise ValueError(f"dest {dest} must be a directory")
@@ -1166,10 +1202,18 @@ class PhotoExporter:
                 )
                 retries += 1
         except Exception as e:
+            _consecutive_export_errors += 1
+            logger.debug(
+                f"AppleScript export error count: {_consecutive_export_errors}"
+            )
             raise ExportError(e)
 
         if not exported_files or not filename:
             # nothing got exported
+            _consecutive_export_errors += 1
+            logger.debug(
+                f"AppleScript export error count: {_consecutive_export_errors}"
+            )
             raise ExportError(f"Could not export photo {uuid} ({lineno(__file__)})")
         # need to find actual filename as sometimes Photos renames JPG to jpeg on export
         # may be more than one file exported (e.g. if Live Photo, Photos exports both .jpeg and .mov)
@@ -1199,6 +1243,11 @@ class PhotoExporter:
                     FileUtil.unlink(dest_new)
                 FileUtil.copy(str(path), str(dest_new))
             exported_paths.append(str(dest_new))
+
+        # Reset error counter on successful export
+        if exported_paths:
+            _consecutive_export_errors = 0
+
         return exported_paths
 
     def _export_aae(
