@@ -1,12 +1,13 @@
-"""Test timezones.py """
+"""Test timezones.py"""
 
 import datetime
+from datetime import timedelta, timezone
 from zoneinfo import ZoneInfo
 
 import pytest
 
 from osxphotos.platform import is_macos
-from osxphotos.timezones import format_offset_time
+from osxphotos.timezones import convert_offset_timezone_to_etc, format_offset_time
 
 if is_macos:
     from osxphotos.timezones import Timezone, known_timezone_names
@@ -73,3 +74,51 @@ def test_format_offset():
     assert format_offset_time(-14400) == "-04:00"
     # Testing UTC +5 hours (in seconds: 5 * 3600 = 18000)
     assert format_offset_time(18000) == "+05:00"
+
+
+@pytest.mark.parametrize(
+    "input_str,expected",
+    [
+        ("GMT-0800", "Etc/GMT+8"),
+        ("GMT+0200", "Etc/GMT-2"),
+        ("GMT+0000", "Etc/GMT+0"),
+        ("GMT-0000", "Etc/GMT+0"),
+        ("utc-0300", "Etc/GMT+3"),
+    ],
+)
+def test_convert_offset_timezone_to_etc_valid_etc_matches(input_str, expected):
+    assert convert_offset_timezone_to_etc(input_str) == expected
+
+
+@pytest.mark.parametrize(
+    "bad_input",
+    [
+        "PST",
+        "GMT+5",  # not HHMM
+        "GMT++0800",  # invalid format
+        "GMT-9999",  # invalid offset
+        "GMT+0530"  # partial hour not handled
+        "GMT-0530",  # partial hour not handled
+    ],
+)
+def test_convert_offset_timezone_to_etc_invalid_format_raises_value_error(bad_input):
+    with pytest.raises(ValueError):
+        convert_offset_timezone_to_etc(bad_input)
+
+
+@pytest.mark.parametrize(
+    "out_of_bounds",
+    [
+        "GMT+1460",  # hours > 14
+        "UTC+1275",  # minutes > 59
+    ],
+)
+def test_convert_offset_timezone_to_etc_invalid_bounds_raises_value_error(
+    out_of_bounds,
+):
+    with pytest.raises(ValueError):
+        convert_offset_timezone_to_etc(out_of_bounds)
+
+
+def test_convert_offset_timezone_to_etc_leading_trailing_whitespace_handled():
+    assert convert_offset_timezone_to_etc("  GMT-0800 ") == "Etc/GMT+8"
